from ftw.casauth.plugin import CASAuthenticationPlugin
from ftw.testbrowser import TRAVERSAL_BROWSER_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig


class FtwCasauthLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.restapi
        xmlconfig.file(
            'configure.zcml',
            plone.restapi,
            context=configurationContext
        )
        import ftw.casauth
        xmlconfig.file(
            'configure.zcml',
            ftw.casauth,
            context=configurationContext
        )
        z2.installProduct(app, 'plone.restapi')
        z2.installProduct(app, 'ftw.casauth')

    def setUpPloneSite(self, portal):

        # Setup PAS plugin
        uf = portal.acl_users
        plugin = CASAuthenticationPlugin(
            'cas_auth', cas_server_url='https://cas.domain.net')
        uf._setObject(plugin.getId(), plugin)
        plugin = uf['cas_auth']
        plugin.manage_activateInterfaces([
            'IAuthenticationPlugin',
            'IChallengePlugin',
            'IExtractionPlugin',
        ])
        self['plugin'] = plugin
        applyProfile(portal, 'plone.restapi:default')


FTW_CASAUTH_FIXTURE = FtwCasauthLayer()
FTW_CASAUTH_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_CASAUTH_FIXTURE, TRAVERSAL_BROWSER_FIXTURE),
    name="FtwcasauthLayer:Integration"
)
FTW_CASAUTH_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_CASAUTH_FIXTURE, z2.ZSERVER_FIXTURE),
    name="FtwcasauthLayer:Functional"
)



import os.path


def get_data(filename):
    """Return content from a file in the test data folder """
    filename = os.path.join(os.path.dirname(__file__), 'data', filename)
    return open(filename, 'r').read()


class MockRequest(object):

    def get_type(self):
        return 'https'


class MockResponse(object):

    def __init__(self, data, code=200, msg='OK'):
        self.data = data
        self.code = code
        self.msg = msg

    def read(self):
        return self.data

    def getcode(self):
        return self.code

    def info(self):
        return {}



"""SSL server certificate verification for `urllib2`.

Python's built-in HTTPS support does not verify certificates.
Thus we provide our own HTTPSHandler which *does* certificate validation.

Example usage::

    >>> opener = urllib2.build_opener(HTTPSHandler)
    >>> opener.open('https://example.com/').read()

"""
from backports.ssl_match_hostname import match_hostname, CertificateError
import httplib
import os
import socket
import ssl
import sys
import urllib2

CERT_FILE = os.path.join(os.path.dirname(__file__), 'cacert.pem')


if not hasattr(socket, 'create_connection'):  # for Python 2.4
    _GLOBAL_DEFAULT_TIMEOUT = getattr(socket,
                                      '_GLOBAL_DEFAULT_TIMEOUT', object())

    # copy-paste from stdlib's socket.py (py2.6)
    def create_connection(address, timeout=_GLOBAL_DEFAULT_TIMEOUT,
                          source_address=None):
        """Connect to *address* and return the socket object.

        Convenience function.  Connect to *address* (a 2-tuple ``(host,
        port)``) and return the socket object.  Passing the optional
        *timeout* parameter will set the timeout on the socket instance
        before attempting to connect.  If no *timeout* is supplied, the
        global default timeout setting returned by :func:`getdefaulttimeout`
        is used.
        """

        host, port = address
        err = None
        for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
            af, socktype, proto, canonname, sa = res
            sock = None
            try:
                sock = socket.socket(af, socktype, proto)
                if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
                    sock.settimeout(timeout)
                if source_address:
                    sock.bind(source_address)
                sock.connect(sa)
                return sock

            except socket.error:
                err = sys.exc_info()[1]
                if sock is not None:
                    sock.close()

        if err is not None:
            raise err
        else:
            raise socket.error("getaddrinfo returns an empty list")

    # monkey-patch socket module
    socket.create_connection = create_connection


class HTTPSConnection(httplib.HTTPConnection):
    "This class allows communication via SSL."

    default_port = httplib.HTTPS_PORT

    def connect(self):
        "Connect to a host on a given (SSL) port."

        if not getattr(self, 'timeout', None):
            self.timeout = socket.getdefaulttimeout()

        args = [(self.host, self.port), self.timeout]
        if hasattr(self, 'source_address'):
            args.append(self.source_address)
        sock = socket.create_connection(*args)

        if getattr(self, '_tunnel_host', None):
            self.sock = sock
            self._tunnel()
        self.sock = ssl.wrap_socket(sock,
                                    ssl_version=ssl.PROTOCOL_TLSv1,
                                    ca_certs=CERT_FILE,
                                    cert_reqs=ssl.CERT_REQUIRED)
        try:
            match_hostname(self.sock.getpeercert(), self.host)
        except CertificateError:
            self.sock.shutdown(socket.SHUT_RDWR)
            self.sock.close()
            raise


class HTTPSHandler(urllib2.HTTPSHandler):

    def https_open(self, req):
            return self.do_open(HTTPSConnection, req)



from AccessControl.Permissions import add_user_folders
from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin
from ftw.casauth import plugin


def initialize(context):
    """Initializer called when used as a Zope 2 product."""
    registerMultiPlugin(plugin.CASAuthenticationPlugin.meta_type)
    context.registerClass(
        plugin.CASAuthenticationPlugin,
        permission=add_user_folders,
        constructors=(plugin.manage_addCASAuthenticationPlugin,
                      plugin.addCASAuthenticationPlugin),
        visibility=None,
    )



import unittest
from ftw.casauth.tests.utils import MockRequest
from ftw.casauth.tests.utils import MockResponse
from ftw.casauth.tests.utils import get_data
from ftw.casauth.cas import validate_ticket
from mock import patch


class TestValdidateTicket(unittest.TestCase):

    @patch('ftw.casauth.cas.HTTPSHandler', autospec=True)
    def test_validate_ticket_suceeds_with_valid_ticket(self, MockHTTPSHandler):
        MockHTTPSHandler.https_request.return_value = MockRequest()
        MockHTTPSHandler.https_open.return_value = MockResponse(
            get_data('service_validate_success.xml'))
        self.assertEqual('james', validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.HTTPSHandler', autospec=True)
    def test_validate_ticket_fails_with_invalid_ticket(self, MockHTTPSHandler):
        MockHTTPSHandler.https_request.return_value = MockRequest()
        MockHTTPSHandler.https_open.return_value = MockResponse(
            get_data('service_validate_invalid_ticket.xml'))
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.HTTPSHandler', autospec=True)
    def test_validate_ticket_fails_with_invalid_response(self, MockHTTPSHandler):
        MockHTTPSHandler.https_request.return_value = MockRequest()
        MockHTTPSHandler.https_open.return_value = MockResponse("Invalid Response")
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.HTTPSHandler', autospec=True)
    def test_validate_ticket_fails_with_invalid_xml_response(self, MockHTTPSHandler):
        MockHTTPSHandler.https_request.return_value = MockRequest()
        MockHTTPSHandler.https_open.return_value = MockResponse("<resp>invalid</resp>")
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))



import sys

# Python 2.7.9 and later supports SSL certificate verification
# Previous versions require a custom HTTPS handler.
if sys.version_info[1] == 7 and sys.version_info[2] >= 9:
    USE_CUSTOM_HTTPS_HANDLER = False
else:
    USE_CUSTOM_HTTPS_HANDLER = True



from ftw.casauth.testing import FTW_CASAUTH_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_ID, TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.testing.z2 import Browser

import unittest
import transaction


class TestCASAuthPlugin(unittest.TestCase):

    layer = FTW_CASAUTH_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        transaction.commit()

    def browser(self, path):
        browser = Browser(self.portal)
        browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD,))
        browser.open(self.portal.absolute_url() + path)
        return browser

    def test_install_plugin(self):
        browser = self.browser('/acl_users/manage_addProduct/ftw.casauth/manage_addCASAuthenticationPlugin')
        self.assertIn('<form action="addCASAuthenticationPlugin"', browser.contents)

        browser.getControl(name="id_").value = 'my_cas_plugin'
        browser.getControl(name="title").value = 'My CAS plugin'
        browser.getControl(name="cas_server_url").value = 'https://mycas'
        browser.getControl('Add').click()
        self.assertIn('my_cas_plugin', browser.contents)
        self.assertIn('My CAS plugin', browser.contents)

        transaction.begin()
        self.assertIn('my_cas_plugin', self.portal.acl_users.objectIds())

        plugin = self.portal.acl_users['my_cas_plugin']
        self.assertEquals('My CAS plugin', plugin.title)
        self.assertEquals('https://mycas', plugin.cas_server_url)

    def test_install_plugin_strips_trailing_slash_of_cas_server_url(self):
        browser = self.browser('/acl_users/manage_addProduct/ftw.casauth/manage_addCASAuthenticationPlugin')
        browser.getControl(name="id_").value = 'my_cas_plugin'
        browser.getControl(name="cas_server_url").value = 'https://mycas/'
        browser.getControl('Add').click()
        transaction.begin()
        self.assertEquals('https://mycas', self.portal.acl_users['my_cas_plugin'].cas_server_url)

    def test_config_updates_cas_server_url(self):
        browser = self.browser('/acl_users/cas_auth/manage_config')
        self.assertIn('<form action="manage_updateConfig"', browser.contents)

        browser.getControl(name="cas_server_url").value = 'https://anothercas'
        browser.getControl('Update').click()
        self.assertIn('Configuration updated', browser.contents)

        transaction.begin()
        self.assertEquals(
            'https://anothercas',
            self.portal.acl_users['cas_auth'].cas_server_url)

    def test_update_config_strips_trailing_slash_of_cas_server_url(self):
        browser = self.browser('/acl_users/cas_auth/manage_config')
        browser.getControl(name="cas_server_url").value = 'https://anothercas/'
        browser.getControl('Update').click()

        transaction.begin()
        self.assertEquals(
            'https://anothercas',
            self.portal.acl_users['cas_auth'].cas_server_url)



from setuptools import setup, find_packages
import os
import sys

version = '1.3.0'

tests_require = [
    'plone.api',
    'plone.app.testing',
    'plone.restapi',
    'mock',
    'ftw.testbrowser',
    'ftw.testing',
]

install_requires = [
    'Plone',
    'setuptools',
]
python_major_version = sys.version_info[0]
python_minor_version = sys.version_info[1]
python_micro_version = sys.version_info[2]
if python_major_version == 2:
    if python_minor_version < 5:
        install_requires.append('hashlib')
    if python_minor_version < 6:
        install_requires.append('ssl')
    if not (python_minor_version == 7 and python_micro_version >= 9):
        install_requires.append('backports.ssl_match_hostname')

setup(
    name='ftw.casauth',
    version=version,
    description='Plone PAS plugin for authentication against CAS.',
    long_description=(open('README.rst').read() + '\n' +
                      open(os.path.join('docs', 'HISTORY.txt')).read()),
    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.1',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='',
    author='Thomas Buchberger',
    author_email='mailto:t.buchberger@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.casauth',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw'],
    include_package_data=True,
    zip_safe=False,

    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=dict(tests=tests_require),
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
